
/*
** Where the default resources are.
*/
#ifndef RESOURCE_PATH
#define RESOURCE_PATH	"./Tuner.resources"
#endif

/*
** Max number of broadcast nets searched, radio stations and
** records listed in playlist.
*/
#define MAXNET 5		/* Maximum number of networks to poll */
#define MAXSTATION 20		/* Maximum number of stations */
#define MAXPLBUF 100		/* Keep 100 last tracks */
#define MAXNAME 64		/* Max size of all sorts of names */

/*
** Networks on which to look for radio stations
*/
#define DFT_NETS	"192.16.184.0", "192.16.191.0", "192.16.201.255"

/*
** Control ports for transmitter and radio programs.
*/
#define CTLPRT_SCALL	54317
#define CTLPRT_XMIT	54319
#define CTLPRT_RADIO	54320

/*
** Timeout (in seconds) waiting for replies to station locate broadcast
*/
#define WAIT_TIMEOUT	2

/*
** Commands to start radio and anp. Remove ANP_COMMAND definition if
** you don't want that functionality.
*/
#define RADIO_COMMAND	"/usr/local/radio/bin/radio"
#define KILLRADIO_COMMAND "/etc/killall radio"
#define ANP_COMMAND	"/ufs/jack/bin/sgi/anp -fromtuner"

/* COMMAND_TRAILER attempts to close some file descriptors on exec */
#define COMMAND_TRAILER "3<&- 4<&- 5<&- 6<&- 7<&- 8<&- 9<&-"
