#include <X11/Intrinsic.h>
#include <Xm/Xm.h>

#include "customize.h"

#ifdef __STDC__
#define ARGS(x) x
#else
#define ARGS(x) ()
#define void int
#endif

#ifndef GLOBAL
#define GLOBAL extern
#endif /* GLOBAL */

GLOBAL int cur_tune;					/* From tuner_io */

GLOBAL Widget toplevel;                             /* Main widget */

GLOBAL Widget form;					   /* The form */

GLOBAL Widget currentStation, currentPlaying, currentTime; /* current station info */
GLOBAL Widget playlistBrowser;                             /* cur playlist */
GLOBAL Widget tunerBrowser;                                /* station selector */
GLOBAL Widget muteButton;                                  /* The buttons */
GLOBAL Widget detailsButton;
GLOBAL Widget revertButton;
GLOBAL Widget listenButton;
GLOBAL Widget okButton;
GLOBAL Widget radioButton;
#ifdef ANP_COMMAND
GLOBAL Widget anpButton;
#endif /* ANP_COMMAND */
#ifdef VOLUME
GLOBAL Widget volumeSlider;
GLOBAL Widget labelVolume;
#endif /* VOLUME */

GLOBAL Widget labelCurrent, labelTuner, labelPlaylist;

GLOBAL XmString *tunerData;

GLOBAL XmString *playlistData;                             /* The playlist */
GLOBAL int playlistDataN;

GLOBAL int origstation;

/* From tuner_main.c */
void do_tune ARGS((int));
void do_mute ARGS((int));
void do_revert ARGS((void));

/* From tuner_ui.c */
void init_ui ARGS((int *, char **));
void show_tune ARGS((int, char *, char *, char *, int));
void show_tuner ARGS((char **, int));
void show_playlist ARGS((char **, int, int));
void show_volume ARGS((int));

/* From tuner_io.c */
#include <netinet/in.h>
void sendsock ARGS((int, struct sockaddr_in *, char *));
char *recvsock ARGS((int));
